# demo 工程说明

### 工程结构

![](gic-demo-helper/src/main/resources/snapshot_01.png)

|模块   |子模块   |说明   |
| ------------ | ------------ | ------------ |
|gic-demo-pom   |-   |基础pom，项目组中只能有一个或是直接继承 gic-pom-base   |
|gic-demo-base   |gic-demo-base-api   |基础api，项目组中只能有一个或是直接继承 gic-base-api   |
|   |gic-demo-common   |常用工具扩展(仅含项目组专用)，项目组中只能有一个或是直接继承 gic-commons   |
|   |gic-demo-config   |常用配置扩展，项目分组中只能有一个或是直接继承 gic-platform-config   |
|   |gic-demo-helper   |代码助手，不限于MyBatis代码生成   |
|gic-demo-project   |gic-demo-project-api   |模块 api  |
|   |gic-demo-project-service   |模块 dubbo 服务   |
|   |gic-demo-project-web   |模块 web rest 服务   |

### 命名规范补充
|包   |说明   |
| ------------ | ------------ |
|qo   |query object， web 查询对象，类名以QO结尾   |
|vo   |view object，web 视图对象，类名以VO结尾   |
|dto   |data transfer object，数据传输对象，以DTO结尾   |


### 生成 mybatis model mapper xml 
gic-demo-helper工程，配置generatorConfig_test.xml。
```xml
<context id="testTables" targetRuntime="MyBatis3">

	<!-- 配置数据库连接 -->
	<jdbcConnection driverClass="com.mysql.jdbc.Driver"
					connectionURL="jdbc:mysql://xxxx:6634/test?characterEncoding=utf-8"
					userId="userId"
					password="password">
	</jdbcConnection>
	
	<!-- 配置Model生成 -->
	<javaModelGenerator targetPackage="com.gic.demo.project.service.entity" targetProject="src/main/java">
		<property name="enableSubPackages" value="true"/>
	</javaModelGenerator>
	
	<!-- 配置 mapper xml 生成 -->
	<sqlMapGenerator targetPackage="mapper" targetProject="src/main/resources">
	    <property name="enableSubPackages" value="true"/>
	</sqlMapGenerator>
	
	<!-- 配置 mapper 对应接口 -->
	<javaClientGenerator type="XMLMAPPER"
						 targetPackage="com.gic.demo.project.service.dao.mapper" targetProject="src/main/java">
		<property name="enableSubPackages" value="true"/>
	</javaClientGenerator>

	<!-- 指定数据库表 -->
	<table tableName="tab_gic_demo_enterprise" enableSelectByExample="false" enableDeleteByExample="false" enableCountByExample="false" enableUpdateByExample="false" selectByExampleQueryId="false"/>
	<table tableName="tab_gic_demo_store" enableSelectByExample="false" enableDeleteByExample="false" enableCountByExample="false" enableUpdateByExample="false" selectByExampleQueryId="false"/>
	
</context>

```
生成代码
```java
public class GeneratorStartUp {
    public static void main(String[] args) {
        try {
            List<String> warnings = new ArrayList<>();
            ClassLoader classloader = Thread.currentThread().getContextClassLoader();
            InputStream is = classloader.getResourceAsStream("generatorConfig_test.xml");
            ConfigurationParser cp = new ConfigurationParser(warnings);
            Configuration config = cp.parseConfiguration(is);
            DefaultShellCallback callback = new DefaultShellCallback(true);
            MyBatisGenerator myBatisGenerator = new MyBatisGenerator(config, callback, warnings);
            myBatisGenerator.generate(null);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
```

### 生成雪花id
手动生成
```java
public class TestRun {
	public static void main(String[] args) {
	    // 生成long型id
	    UniqueIdUtils.uniqueLong();
	    // 生成hex字符串型id
	    UniqueIdUtils.uniqueLongHex();
	}
}
```
注解生成
```java
public class TabGicDemoStore {
    /**
     * 门店id
     */
    @InjectId(InjectId.IdType.SNOWFLAKE)
    private String storeId;
}
```
```xml
<bean class="org.mybatis.spring.SqlSessionFactoryBean" id="sqlSessionFactory">
    <property name="plugins">
        <array>
            <!-- 自动生成id -->
            <bean class="com.gic.commons.plugin.mybatis.AutoIdInterceptor"/>
        </array>
    </property>
</bean>
```

### 分页查询
```java
public class XXXXInnerServiceImpl implements XXXXService {

    @Autowired
    private XXXXMapper mapper;

    /**
     * 分页查询
     */
    @Override
    public Page<XXXXDTO> queryPageList(String storeId) {
        // 分页
        PageHelper.startPage(1, 20);
        Page<XXXX> page = mapper.queryPageList(storeId);
        // 转换为DTO
        return PageUtil.changePageHelperToCurrentPage(page, XXXXDTO.class);
    }
}
```

### sharding-jdbc 配置
1. 参照 sharding-jdbc 配置说明配置分表，保存到mongodb
2. 修改 Apollo配置 COMMON.sharding sharding.shardingId
3. xml配置 

```xml
<bean id="dataSource" class="com.gic.sharding.sdk.ShardingDatasource" init-method="init">
    <property name="shardingId" value="${sharding.shardingId}"/>
    <property name="maxSize" value="${jdbc.maxsize}"/>
    <property name="urlType" value="${sharding.urlType:inner}"/>
</bean>
```

### dubbo 服务
参考 gic 3.x，com.alibaba.dubbo.container.Main 替换为 com.gic.commons.DubboMain 。

### web rest 服务
使用 spring boot 内嵌 tomcat 启动
```java
/**
 * Spring Boot Web 启动类
 *
 * @author zhurz
 */
@SpringBootApplication
@ImportResource(value = {
        // spring 配置初始化xml
        "classpath*:gic-demo-init.xml",
        // 通用 dubbo 配置
        "classpath*:dubbo-setting.xml",
        // dubbo api 配置
        "classpath*:dubbo-gic-demo-web.xml",
        // 拦截器配置
        "classpath*:spring-interceptor.xml"
})
public class Main {
    public static void main(String[] args) {
        ConfigurableApplicationContext context = SpringApplication.run(Main.class, args);
        DubboContextUtil.setApplicationContext(context.getParent());
    }
}
```

